@@ -6,7 +6,9 @@ package app
 import (
 	"context"
 	"fmt"
+	"os"
 	"sort"
+	"strings"
 	"time"
 
 	"github.com/k14s/kapp/pkg/kapp/logger"
@@ -20,12 +22,16 @@ import (
 )
 
 const (
-	kappAppLabelKey = "kapp.k14s.io/app"
+	kappAppLabelKey                        = "kapp.k14s.io/app"
+	KappIsConfigmapMigratedAnnotationKey   = "kapp.k14s.io/is-configmap-migrated"
+	KappIsConfigmapMigratedAnnotationValue = ""
+	AppSuffix                              = ".apps.k14s.io"
 )
 
 type RecordedApp struct {
 	name              string
 	nsName            string
+	isMigrated        bool
 	creationTimestamp time.Time
 
 	coreClient             kubernetes.Interface
@@ -36,11 +42,21 @@ type RecordedApp struct {
 	logger       logger.Logger
 }
 
+func NewRecordedApp(name, nsName string, creationTimestamp time.Time, coreClient kubernetes.Interface,
+	identifiedResources ctlres.IdentifiedResources, appInDiffNsHintMsgFunc func(string) string, logger logger.Logger) *RecordedApp {
+
+	// Always trim suffix, even if user added it manually (to avoid double migration)
+	return &RecordedApp{strings.TrimSuffix(name, AppSuffix), nsName, false, creationTimestamp, coreClient, identifiedResources, appInDiffNsHintMsgFunc,
+		nil, logger.NewPrefixed("RecordedApp")}
+}
+
 var _ App = &RecordedApp{}
 
 func (a *RecordedApp) Name() string      { return a.name }
 func (a *RecordedApp) Namespace() string { return a.nsName }
 
+func (a *RecordedApp) fqName() string { return a.name + AppSuffix }
+
 func (a *RecordedApp) CreationTimestamp() time.Time { return a.creationTimestamp }
 
 func (a *RecordedApp) Description() string {
@@ -114,6 +130,43 @@ func (a *RecordedApp) UpdateUsedGVsAndGKs(gvs []schema.GroupVersion, gks []schem
 func (a *RecordedApp) CreateOrUpdate(labels map[string]string) error {
 	defer a.logger.DebugFunc("CreateOrUpdate").Finish()
 
+	app, foundMigratedApp, err := a.find(a.fqName())
+	if err != nil {
+		return err
+	}
+
+	if foundMigratedApp {
+		a.isMigrated = true
+		return a.updateApp(app, labels)
+	}
+
+	app, foundNonMigratedApp, err := a.find(a.name)
+	if err != nil {
+		return err
+	}
+
+	if foundNonMigratedApp {
+		if a.isMigrationEnabled() {
+			return a.migrate(app, labels, a.fqName())
+		}
+		return a.updateApp(app, labels)
+	}
+
+	return a.create(labels)
+}
+
+func (a *RecordedApp) find(name string) (*corev1.ConfigMap, bool, error) {
+	cm, err := a.coreClient.CoreV1().ConfigMaps(a.nsName).Get(context.TODO(), name, metav1.GetOptions{})
+	if err != nil {
+		if errors.IsNotFound(err) {
+			return nil, false, nil
+		}
+		return nil, false, fmt.Errorf("Getting app: %w", err)
+	}
+	return cm, true, nil
+}
+
+func (a *RecordedApp) create(labels map[string]string) error {
 	configMap := &corev1.ConfigMap{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      a.name,
@@ -129,35 +182,113 @@ func (a *RecordedApp) CreateOrUpdate(labels map[string]string) error {
 		}.AsData(),
 	}
 
+	if a.isMigrationEnabled() {
+		configMap.ObjectMeta.Name = a.fqName()
+		a.isMigrated = true
+
+		err := a.mergeAppAnnotationUpdates(configMap, map[string]string{KappIsConfigmapMigratedAnnotationKey: KappIsConfigmapMigratedAnnotationValue})
+		if err != nil {
+			return err
+		}
+	}
+
 	err := a.mergeAppUpdates(configMap, labels)
 	if err != nil {
 		return err
 	}
 
 	_, err = a.coreClient.CoreV1().ConfigMaps(a.nsName).Create(context.TODO(), configMap, metav1.CreateOptions{})
+
+	return err
+}
+
+func (a *RecordedApp) updateApp(existingConfigMap *corev1.ConfigMap, labels map[string]string) error {
+	err := a.mergeAppUpdates(existingConfigMap, labels)
 	if err != nil {
-		if errors.IsAlreadyExists(err) {
-			existingConfigMap, err := a.coreClient.CoreV1().ConfigMaps(a.nsName).Get(context.TODO(), a.name, metav1.GetOptions{})
-			if err != nil {
-				return fmt.Errorf("Getting app: %s", err)
-			}
-
-			err = a.mergeAppUpdates(existingConfigMap, labels)
-			if err != nil {
-				return err
-			}
-
-			_, err = a.coreClient.CoreV1().ConfigMaps(a.nsName).Update(context.TODO(), existingConfigMap, metav1.UpdateOptions{})
-			if err != nil {
-				return fmt.Errorf("Updating app: %s", err)
-			}
-
-			return nil
-		}
-
-		return fmt.Errorf("Creating app: %s", err)
+		return err
 	}
 
+	_, err = a.coreClient.CoreV1().ConfigMaps(a.nsName).Update(context.TODO(), existingConfigMap, metav1.UpdateOptions{})
+	if err != nil {
+		return fmt.Errorf("Updating app: %s", err)
+	}
+
+	return nil
+}
+
+func (a *RecordedApp) RenamePrevApp(prevAppName string, labels map[string]string) error {
+	defer a.logger.DebugFunc("RenamePrevApp").Finish()
+
+	app, foundMigratedApp, err := a.find(a.fqName())
+	if err != nil {
+		return err
+	}
+
+	if foundMigratedApp {
+		a.isMigrated = true
+		return a.updateApp(app, labels)
+	}
+
+	app, foundNonMigratedApp, err := a.find(a.name)
+	if err != nil {
+		return err
+	}
+
+	if foundNonMigratedApp {
+		if a.isMigrationEnabled() {
+			return a.migrate(app, labels, a.fqName())
+		}
+		return a.updateApp(app, labels)
+	}
+
+	app, foundMigratedPrevApp, err := a.find(prevAppName + AppSuffix)
+	if err != nil {
+		return err
+	}
+
+	if foundMigratedPrevApp {
+		a.isMigrated = true
+		return a.renameConfigMap(app, a.fqName(), a.nsName)
+	}
+
+	app, foundNonMigratedPrevApp, err := a.find(prevAppName)
+	if err != nil {
+		return err
+	}
+
+	if foundNonMigratedPrevApp {
+		if a.isMigrationEnabled() {
+			return a.migrate(app, labels, a.fqName())
+		}
+		return a.renameConfigMap(app, a.name, a.nsName)
+	}
+
+	return a.create(labels)
+}
+
+func (a *RecordedApp) migrate(c *corev1.ConfigMap, labels map[string]string, newName string) error {
+	err := a.renameConfigMap(c, newName, a.nsName)
+	if err != nil {
+		return err
+	}
+
+	err = a.mergeAppUpdates(c, labels)
+	if err != nil {
+		return err
+	}
+
+	err = a.mergeAppAnnotationUpdates(c, map[string]string{KappIsConfigmapMigratedAnnotationKey: KappIsConfigmapMigratedAnnotationValue})
+	if err != nil {
+		return err
+	}
+
+	_, err = a.coreClient.CoreV1().ConfigMaps(a.nsName).Update(context.TODO(), c, metav1.UpdateOptions{})
+	if err != nil {
+		return fmt.Errorf("Updating app: %s", err)
+	}
+
+	a.isMigrated = true
+
 	return nil
 }
 
@@ -174,15 +305,43 @@ func (a *RecordedApp) mergeAppUpdates(cm *corev1.ConfigMap, labels map[string]st
 	return nil
 }
 
-func (a *RecordedApp) Exists() (bool, string, error) {
-	_, err := a.coreClient.CoreV1().ConfigMaps(a.nsName).Get(context.TODO(), a.name, metav1.GetOptions{})
-	if err != nil {
-		if errors.IsNotFound(err) {
-			desc := fmt.Sprintf("App '%s' (namespace: %s) does not exist%s",
-				a.name, a.nsName, a.appInDiffNsHintMsgFunc(a.name))
-			return false, desc, nil
+func (a *RecordedApp) mergeAppAnnotationUpdates(cm *corev1.ConfigMap, annotations map[string]string) error {
+	if cm.ObjectMeta.Annotations == nil && len(annotations) > 0 {
+		cm.ObjectMeta.Annotations = map[string]string{}
+	}
+
+	for key, val := range annotations {
+		if prevVal, found := cm.ObjectMeta.Annotations[key]; found {
+			if prevVal != val {
+				return fmt.Errorf("Expected annotation '%s' value to remain same", key)
+			}
 		}
-		return false, "", fmt.Errorf("Getting app: %s", err)
+		cm.ObjectMeta.Annotations[key] = val
+	}
+
+	return nil
+}
+
+func (a *RecordedApp) Exists() (bool, string, error) {
+	_, foundMigratedApp, err := a.find(a.fqName())
+	if err != nil {
+		return false, "", err
+	}
+
+	if foundMigratedApp {
+		a.isMigrated = true
+		return true, "", nil
+	}
+
+	_, foundNonMigratedApp, err := a.find(a.name)
+	if err != nil {
+		return false, "", err
+	}
+
+	if !foundNonMigratedApp {
+		desc := fmt.Sprintf("App '%s' (namespace: %s) does not exist%s",
+			a.name, a.nsName, a.appInDiffNsHintMsgFunc(a.name))
+		return false, desc, nil
 	}
 
 	return true, "", nil
@@ -204,7 +363,12 @@ func (a *RecordedApp) Delete() error {
 		return err
 	}
 
-	err = a.coreClient.CoreV1().ConfigMaps(a.nsName).Delete(context.TODO(), a.name, metav1.DeleteOptions{})
+	name := a.name
+	if a.isMigrated {
+		name = a.fqName()
+	}
+
+	err = a.coreClient.CoreV1().ConfigMaps(a.nsName).Delete(context.TODO(), name, metav1.DeleteOptions{})
 	if err != nil {
 		return fmt.Errorf("Deleting app: %s", err)
 	}
@@ -213,29 +377,47 @@ func (a *RecordedApp) Delete() error {
 }
 
 func (a *RecordedApp) Rename(newName string, newNamespace string) error {
-	app, err := a.coreClient.CoreV1().ConfigMaps(a.nsName).Get(context.TODO(), a.name, metav1.GetOptions{})
-	if err != nil {
-		if errors.IsNotFound(err) {
-			return fmt.Errorf("App '%s' (namespace: %s) does not exist: %s%s",
-				a.name, a.nsName, err, a.appInDiffNsHintMsgFunc(a.name))
-		}
-		return fmt.Errorf("Getting app: %s", err)
+	name := a.name
+	if a.isMigrated {
+		name = a.fqName()
 	}
 
+	app, found, err := a.find(name)
+	if err != nil {
+		return err
+	}
+
+	if !found {
+		return fmt.Errorf("App '%s' (namespace: %s) does not exist: %s",
+			a.name, a.nsName, a.appInDiffNsHintMsgFunc(name))
+	}
+
+	// use fully qualified name if app had been previously migrated
+	if a.isMigrated || a.isMigrationEnabled() {
+		a.mergeAppAnnotationUpdates(app, map[string]string{KappIsConfigmapMigratedAnnotationKey: KappIsConfigmapMigratedAnnotationValue})
+		return a.renameConfigMap(app, newName+AppSuffix, newNamespace)
+	}
+
+	return a.renameConfigMap(app, newName, newNamespace)
+}
+
+func (a *RecordedApp) renameConfigMap(app *corev1.ConfigMap, name, ns string) error {
+	oldName := app.Name
+
 	// Clear out all existing meta fields
 	app.ObjectMeta = metav1.ObjectMeta{
-		Name:        newName,
-		Namespace:   newNamespace,
+		Name:        name,
+		Namespace:   ns,
 		Labels:      app.ObjectMeta.Labels,
 		Annotations: app.ObjectMeta.Annotations,
 	}
 
-	_, err = a.coreClient.CoreV1().ConfigMaps(newNamespace).Create(context.TODO(), app, metav1.CreateOptions{})
+	_, err := a.coreClient.CoreV1().ConfigMaps(ns).Create(context.TODO(), app, metav1.CreateOptions{})
 	if err != nil {
 		return fmt.Errorf("Creating app: %s", err)
 	}
 
-	err = a.coreClient.CoreV1().ConfigMaps(a.nsName).Delete(context.TODO(), a.name, metav1.DeleteOptions{})
+	err = a.coreClient.CoreV1().ConfigMaps(a.nsName).Delete(context.TODO(), oldName, metav1.DeleteOptions{})
 	if err != nil {
 		// TODO Do not clean up new config map as there is no gurantee it can be deleted either
 		return fmt.Errorf("Deleting app: %s", err)
@@ -257,6 +439,10 @@ func (a *RecordedApp) labeledApp() (*LabeledApp, error) {
 	return &LabeledApp{sel, a.identifiedResources}, nil
 }
 
+func (a *RecordedApp) isMigrationEnabled() bool {
+	return strings.ToLower(os.Getenv("KAPP_FQ_CONFIGMAP_NAMES")) == "true"
+}
+
 func (a *RecordedApp) Meta() (Meta, error) { return a.meta() }
 
 func (a *RecordedApp) setMeta(app corev1.ConfigMap) (Meta, error) {
@@ -264,6 +450,10 @@ func (a *RecordedApp) setMeta(app corev1.ConfigMap) (Meta, error) {
 	if err != nil {
 		errMsg := "App '%s' (namespace: %s) backed by ConfigMap '%s' did not contain parseable app metadata: %s"
 		hintText := " (hint: ConfigMap was overriden by another user?)"
+
+		if a.isMigrated {
+			return Meta{}, fmt.Errorf(errMsg+hintText, a.name, a.nsName, a.fqName(), err)
+		}
 		return Meta{}, fmt.Errorf(errMsg+hintText, a.name, a.nsName, a.name, err)
 	}
 
@@ -278,13 +468,24 @@ func (a *RecordedApp) meta() (Meta, error) {
 		return *a.memoizedMeta, nil
 	}
 
-	app, err := a.coreClient.CoreV1().ConfigMaps(a.nsName).Get(context.TODO(), a.name, metav1.GetOptions{})
+	app, foundMigratedApp, err := a.find(a.fqName())
 	if err != nil {
-		if errors.IsNotFound(err) {
-			return Meta{}, fmt.Errorf("App '%s' (namespace: %s) does not exist: %s%s",
-				a.name, a.nsName, err, a.appInDiffNsHintMsgFunc(a.name))
-		}
-		return Meta{}, fmt.Errorf("Getting app: %s", err)
+		return Meta{}, err
+	}
+
+	if foundMigratedApp {
+		a.isMigrated = true
+		return a.setMeta(*app)
+	}
+
+	app, foundNonMigratedApp, err := a.find(a.name)
+	if err != nil {
+		return Meta{}, err
+	}
+
+	if !foundNonMigratedApp {
+		return Meta{}, fmt.Errorf("App '%s' (namespace: %s) does not exist: %s",
+			a.name, a.nsName, a.appInDiffNsHintMsgFunc(a.name))
 	}
 
 	return a.setMeta(*app)
@@ -332,7 +533,12 @@ func (a *RecordedApp) BeginChange(meta ChangeMeta) (Change, error) {
 }
 
 func (a *RecordedApp) update(doFunc func(*Meta)) error {
-	change, err := a.coreClient.CoreV1().ConfigMaps(a.nsName).Get(context.TODO(), a.name, metav1.GetOptions{})
+	name := a.name
+	if a.isMigrated {
+		name = a.fqName()
+	}
+
+	change, err := a.coreClient.CoreV1().ConfigMaps(a.nsName).Get(context.TODO(), name, metav1.GetOptions{})
 	if err != nil {
 		return fmt.Errorf("Getting app: %s", err)
 	}
