@@ -4,6 +4,7 @@
 package clusterapply
 
 import (
+	ctlconf "github.com/k14s/kapp/pkg/kapp/config"
 	ctldiff "github.com/k14s/kapp/pkg/kapp/diff"
 	ctlres "github.com/k14s/kapp/pkg/kapp/resources"
 )
@@ -15,6 +16,7 @@ type ClusterChangeFactory struct {
 	changeSetFactory    ctldiff.ChangeSetFactory
 	convergedResFactory ConvergedResourceFactory
 	ui                  UI
+	diffMaskRules       []ctlconf.DiffMaskRule
 }
 
 func NewClusterChangeFactory(
@@ -22,13 +24,14 @@ func NewClusterChangeFactory(
 	identifiedResources ctlres.IdentifiedResources,
 	changeFactory ctldiff.ChangeFactory,
 	changeSetFactory ctldiff.ChangeSetFactory,
-	convergedResFactory ConvergedResourceFactory, ui UI,
+	convergedResFactory ConvergedResourceFactory,
+	ui UI, diffMaskRules []ctlconf.DiffMaskRule,
 ) ClusterChangeFactory {
 	return ClusterChangeFactory{opts, identifiedResources,
-		changeFactory, changeSetFactory, convergedResFactory, ui}
+		changeFactory, changeSetFactory, convergedResFactory, ui, diffMaskRules}
 }
 
 func (f ClusterChangeFactory) NewClusterChange(change ctldiff.Change) *ClusterChange {
 	return NewClusterChange(change, f.opts, f.identifiedResources,
-		f.changeFactory, f.changeSetFactory, f.convergedResFactory, f.ui)
+		f.changeFactory, f.changeSetFactory, f.convergedResFactory, f.ui, f.diffMaskRules)
 }
