@@ -96,7 +96,12 @@ func (o *DeployOptions) Run() error {
 		return err
 	}
 
-	err = app.CreateOrUpdate(appLabels)
+	if o.DeployFlags.PrevApp != "" {
+		err = app.RenamePrevApp(o.DeployFlags.PrevApp, appLabels)
+	} else {
+		err = app.CreateOrUpdate(appLabels)
+	}
+
 	if err != nil {
 		return err
 	}
@@ -401,7 +406,7 @@ func (o *DeployOptions) calculateAndPresentChanges(existingResources,
 
 		clusterChangeFactory := ctlcap.NewClusterChangeFactory(
 			o.ApplyFlags.ClusterChangeOpts, supportObjs.IdentifiedResources,
-			changeFactory, changeSetFactory, convergedResFactory, msgsUI)
+			changeFactory, changeSetFactory, convergedResFactory, msgsUI, conf.DiffMaskRules())
 
 		clusterChangeSet = ctlcap.NewClusterChangeSet(
 			changes, o.ApplyFlags.ClusterChangeSetOpts, clusterChangeFactory,
