@@ -13,6 +13,8 @@ type ResourceTypesFlags struct {
 	CanIgnoreFailingAPIService func(schema.GroupVersion) bool
 
 	ScopeToFallbackAllowedNamespaces bool
+
+	DisableGKScoping bool
 }
 
 func (s *ResourceTypesFlags) Set(cmd *cobra.Command) {
@@ -21,6 +23,9 @@ func (s *ResourceTypesFlags) Set(cmd *cobra.Command) {
 
 	cmd.Flags().BoolVar(&s.ScopeToFallbackAllowedNamespaces, "dangerous-scope-to-fallback-allowed-namespaces",
 		false, "Scope resource searching to fallback allowed namespaces")
+
+	cmd.Flags().BoolVar(&s.DisableGKScoping, "dangerous-disable-gk-scoping",
+		false, "Disable scoping of resource searching to used GroupKinds")
 }
 
 func (s *ResourceTypesFlags) FailingAPIServicePolicy() *FailingAPIServicesPolicy {
