@@ -4,8 +4,8 @@
 package diffgraph_test
 
 import (
-	"testing"
 	"strings"
+	"testing"
 
 	ctldgraph "github.com/k14s/kapp/pkg/kapp/diffgraph"
 	"github.com/stretchr/testify/require"
@@ -22,14 +22,14 @@ func TestNewChangeGroupFromAnnString(t *testing.T) {
 		"valid-name.com/valid-name_Another_Name--valid",
 		"valid-name.com/valid-name_CustomResourceDefinition--valid",
 		// Allow arbitrary long names since it might be populated with data via placeholders
-		"valid-name.com/valid-name_CustomResourceDefinition--valid"+strings.Repeat("a", 1000),
+		"valid-name.com/valid-name_CustomResourceDefinition--valid" + strings.Repeat("a", 1000),
 		// Example from pinniped of a long name
 		"change-groups.kapp.k14s.io/crds-authentication.concierge.pinniped.dev-WebhookAuthenticator",
 	}
 	for _, name := range names {
 		cg, err := ctldgraph.NewChangeGroupFromAnnString(name)
 		require.NoError(t, err)
-		require.Equal(t, cg.Name, name)
+		require.Equal(t, name, cg.Name)
 	}
 
 	names = []string{
