@@ -54,9 +54,9 @@ metadata:
 		expectedOutput := `
 Changes
 
-Namespace  Name         Kind       Conds.  Age  Op      Op st.  Wait to    Rs  Ri  $
-(cluster)  external     Namespace  -       -    exists  -       reconcile  -   -  $
-external   kapp-config  ConfigMap  -       -    create  -       reconcile  -   -  $
+Namespace  Name         Kind       Age  Op      Op st.  Wait to    Rs  Ri  $
+(cluster)  external     Namespace  -    exists  -       reconcile  -   -  $
+external   kapp-config  ConfigMap  -    create  -       reconcile  -   -  $
 
 Op:      1 create, 0 delete, 0 update, 0 noop, 1 exists
 Wait to: 2 reconcile, 0 delete, 0 noop
@@ -88,7 +88,7 @@ Succeeded`
 		expectedOutput := `
 Changes
 
-Namespace  Name  Kind  Conds.  Age  Op  Op st.  Wait to  Rs  Ri  $
+Namespace  Name  Kind  Age  Op  Op st.  Wait to  Rs  Ri  $
 
 Op:      0 create, 0 delete, 0 update, 0 noop, 0 exists
 Wait to: 0 reconcile, 0 delete, 0 noop
@@ -108,8 +108,8 @@ Succeeded`
 		expectedOutput := `
 Resources in app 'app'
 
-Namespace  Name         Kind       Owner  Conds.  Rs  Ri  Age  $
-external   kapp-config  ConfigMap  kapp   -       ok  -   <replaced>  $
+Namespace  Name         Kind       Owner  Rs  Ri  Age  $
+external   kapp-config  ConfigMap  kapp   ok  -   <replaced>  $
 
 Rs: Reconcile state
 Ri: Reconcile information
