@@ -4,8 +4,9 @@
 package e2e
 
 import (
-	"strings"
 	"testing"
+
+	"github.com/stretchr/testify/require"
 )
 
 func TestHelpCommandGroup(t *testing.T) {
@@ -13,10 +14,7 @@ func TestHelpCommandGroup(t *testing.T) {
 	kapp := Kapp{t, env.Namespace, env.KappBinaryPath, Logger{}}
 
 	_, err := kapp.RunWithOpts([]string{"app-group"}, RunOpts{NoNamespace: true, AllowError: true})
-	if err == nil {
-		t.Fatalf("Expected error")
-	}
-	if !strings.Contains(err.Error(), "Error: Use one of available subcommands: delete, deploy") {
-		t.Fatalf("Expected helpful error message, but was '%s'", err.Error())
-	}
+	require.Errorf(t, err, "Expected to receive error")
+
+	require.Contains(t, err.Error(), "Error: Use one of available subcommands: delete, deploy", "Expected helpful error message")
 }
